#include "gbahearts.h"

// menu system
struct MenuData *globalMenuData;
u8 globalCurrentMenu;
u8 globalCurrentMenuItem;
u8 globalMenuStatus;	// allow for transitions
u8 globalMenuTransType;
u8 globalMenuTemp;		// also for transitions
u8 globalItemAnimationStep;

typedef void (*Player_callfunc)(void);

void SetupMenuPal(void) {
	u32 i;
	for(i=0;i<16;i++) {
		SPRPAL[i]=0;
	}
};
void InitScrn(void) {
	globalCurrentMenuItem=globalMenuData->menulist[globalCurrentMenu]->selitem;
	killOam();
	SetupMenuPal();

}

void InitMenu(struct MenuData * mymenu) {
	globalMenuData=mymenu;
	globalCurrentMenu=mymenu->mainmenu;
	InitScrn();
	SetOamZoom(0,256);

}

const u16 MenuPalSelAnim[32]={
	4,4,5,6,7,8,9,11,14,18,22,25,27,28,29,30,
	31,30,29,28,27,25,22,18,14,11,9,8,7,6,5,4 
};
void MenuPalAnim(u8 animframe) {
	u32 i;
	for(i=0;i<16;i++) {
		SPRPAL[16+i]=MenuPalSelAnim[((animframe/2)+i)%32];
	}
}


void MenuOamSet(struct MenuScrn * scrn) {
	u32 i,j,s;
	u32 x,y;
	s=0;
	MenuPalAnim(globalItemAnimationStep++);
	struct MenuItm * itm;
	for(i=0;i<scrn->numitems;i++) {
		itm=scrn->itemlist[i];
		x=itm->x;
		y=itm->y;
		for(j=0;j<itm->textlen;j++) {
			OamA(s,0)=y | 0x0300;
			OamA(s,1)=x;
			OamA(s,2)=(512+itm->text[j]) | ((i==globalCurrentMenuItem)?0x1000:0);
			x+=itm->pitch;
			s++;
		}
	}
}
void DoAction(u8 action,u8 param) {
	switch(action) {
	case 0:
		// do nothing
		break;
	case 1:	// select item
		globalCurrentMenuItem=param;
		PlayGBCSound(globalMenuData->menulist[globalCurrentMenu]->sndcng);

		break;
	case 2:	// change menu
		globalMenuTemp=param;
		globalMenuStatus=64;
		globalMenuTransType=0;
		PlayGBCSound(globalMenuData->menulist[globalCurrentMenu]->sndsel);

		break;
	case 3:	// play sound
		PlayGBCSound(param);
		break;
	case 4:	// call routine
		((Player_callfunc)(globalMenuData->functionlist[param]))();
		break;
	case 5: // transition to self and call routine
		globalMenuTemp=param;
		globalMenuStatus=64;
		globalMenuTransType=1;
		PlayGBCSound(globalMenuData->menulist[globalCurrentMenu]->sndsel);

		break;
	}

}

u32 menuTrans(u32 step) {
	// calculate wierd transition :)
	if(step<8) {
		return 512-((8-step)*(8-step))*4;
	} else {
		if(4*(step-8)*(step-8)>512) return 0; else
		return 512-((step-8)*(step-8))*4;

	}
}
void UpdateMenu(void) {
	struct MenuScrn * scrn;
	u32 keys;
	scrn=globalMenuData->menulist[globalCurrentMenu];

	MenuOamSet(scrn);

	if(globalMenuStatus>0) {
		// transition in progress...
		globalMenuStatus--;
		if(globalMenuStatus>=32) {
			SetOamZoom(0,menuTrans(-globalMenuStatus+64));
		} else {
			SetOamZoom(0,menuTrans(globalMenuStatus));
		}
		if(globalMenuStatus==32) {
			if(globalMenuTransType==0) {
				globalCurrentMenu=globalMenuTemp;
				InitScrn();
			} else {
				((Player_callfunc)(globalMenuData->functionlist[globalMenuTemp]))();
			}
		}
		return;
	}

	keys=GetKeys();

	if(keys & 0x0200) {	// L
		DoAction(scrn->action.L,scrn->paramater.L);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.L,scrn->itemlist[globalCurrentMenuItem]
			->paramater.L);
	}
	if(keys & 0x0100) {	// R
		DoAction(scrn->action.R,scrn->paramater.R);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.R,scrn->itemlist[globalCurrentMenuItem]
			->paramater.R);
	}
	if(keys & 0x0080) {	// dn
		DoAction(scrn->action.dn,scrn->paramater.dn);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.dn,scrn->itemlist[globalCurrentMenuItem]
			->paramater.dn);
	}
	if(keys & 0x0040) {	// up
		DoAction(scrn->action.up,scrn->paramater.up);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.up,scrn->itemlist[globalCurrentMenuItem]
			->paramater.up);
	}
	if(keys & 0x0020) {	// lft
		DoAction(scrn->action.lft,scrn->paramater.lft);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.lft,scrn->itemlist[globalCurrentMenuItem]
			->paramater.lft);
	}
	if(keys & 0x0010) {	// rt
		DoAction(scrn->action.rt,scrn->paramater.rt);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.rt,scrn->itemlist[globalCurrentMenuItem]
			->paramater.rt);
	}
	if(keys & 0x0008) {	// start
		DoAction(scrn->action.st,scrn->paramater.st);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.st,scrn->itemlist[globalCurrentMenuItem]
			->paramater.st);
	}
	if(keys & 0x0004) {	// sel
		DoAction(scrn->action.sl,scrn->paramater.sl);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.sl,scrn->itemlist[globalCurrentMenuItem]
			->paramater.sl);
	}
	if(keys & 0x0002) {	// B
		DoAction(scrn->action.B,scrn->paramater.B);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.B,scrn->itemlist[globalCurrentMenuItem]
			->paramater.B);
	}
	if(keys & 0x0001) {	// A
		DoAction(scrn->action.A,scrn->paramater.A);
		DoAction(scrn->itemlist[globalCurrentMenuItem]
			->action.A,scrn->itemlist[globalCurrentMenuItem]
			->paramater.A);
	}


}






